@@ -32,7 +32,7 @@ GWEN_INHERIT(GWEN_MSGENGINE, AB_MSGENGINE)
 
 
 
-GWEN_MSGENGINE *AB_MsgEngine_new(){
+GWEN_MSGENGINE *AB_MsgEngine_new(void){
   GWEN_MSGENGINE *e;
   AB_MSGENGINE *le;
 
