@@ -129,9 +129,9 @@ uint32_t AH_User_Flags_fromDb(GWEN_DB_NODE *db, const char *name) {
 
 
 
-void AH_User_Extend(AB_USER *u, AB_PROVIDER *pro,
-		    AB_PROVIDER_EXTEND_MODE em,
-		    GWEN_DB_NODE *db) {
+int AH_User_Extend(AB_USER *u, AB_PROVIDER *pro,
+                   AB_PROVIDER_EXTEND_MODE em,
+                   GWEN_DB_NODE *db) {
   DBG_INFO(AQHBCI_LOGDOMAIN, "Extending user with mode %d", em);
   if (em==AB_ProviderExtendMode_Create ||
       em==AB_ProviderExtendMode_Extend) {
@@ -172,16 +172,21 @@ void AH_User_Extend(AB_USER *u, AB_PROVIDER *pro,
       rv=AH_HBCI_UpdateDbUser(ue->hbci, db);
       if (rv<0) {
 	DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not update user db (%d)", rv);
-	assert(0);
+        GWEN_Gui_ShowError(I18N("AqBanking Settings Database Error"),
+                           I18N("Your settings database might be in an inconsistent state!"));
+        return rv;
       }
       AH_User_ReadDb(u, db);
       AH_User_LoadTanMethods(u);
       if (rv==1) {
 	/* updated config, write it now */
+        DBG_NOTICE(AQHBCI_LOGDOMAIN, "Writing back updated HBCI user %d", AB_User_GetUniqueId(u));
 	rv=AB_Banking_SaveUserConfig(AB_Provider_GetBanking(pro), u, 1);
 	if (rv<0) {
-	  DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not save user db (%d)", rv);
-	  assert(0);
+          DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not save user db (%d)", rv);
+          GWEN_Gui_ShowError(I18N("AqBanking Settings Database Error"),
+                             I18N("Your settings database might be in an inconsistent state!"));
+          return rv;
 	}
       }
     }
@@ -203,6 +208,8 @@ void AH_User_Extend(AB_USER *u, AB_PROVIDER *pro,
       AH_User_toDb(u, db);
     } /* if save */
   }
+
+  return 0;
 }
 
 
@@ -1356,10 +1363,11 @@ int AH_User_InputTanWithChallenge(AB_USER *u,
 				  int minLen,
                                   int maxLen){
   int rv;
-  char buffer[512];
+  char buffer[1024];
   const char *un;
   const char *bn=NULL;
   GWEN_BUFFER *nbuf;
+  GWEN_BUFFER *xbuf;
   AB_BANKINFO *bi;
 
   assert(u);
@@ -1376,38 +1384,56 @@ int AH_User_InputTanWithChallenge(AB_USER *u,
 
   buffer[0]=0;
   buffer[sizeof(buffer)-1]=0;
+
+  xbuf=GWEN_Buffer_new(0, 256, 0, 1);
+
+  /* text version */
   snprintf(buffer, sizeof(buffer)-1,
-	   I18N("Please enter the TAN\n"
-		"for user %s at %s.\n"
-                "The server provided the following challenge:\n"
-                "%s"
-                "<html>"
-                "<p>"
-		"Please enter the TAN for user <i>%s</i> at "
-		"<i>%s</i>."
-		"</p>"
-                "<p>"
-                "The server provided the following challenge:"
-                "</p>"
-                "<p align=\"center\" >"
-                "<font color=\"blue\">%s</font>"
-                "</p>"
-                "</html>"),
-	   un, bn, challenge,
-	   un, bn, challenge);
+           I18N("Please enter the TAN\n"
+                "for user %s at %s.\n"), un, bn);
+  buffer[sizeof(buffer)-1]=0;
+  GWEN_Buffer_AppendString(xbuf, buffer);
+  if (challenge && *challenge) {
+    GWEN_Buffer_AppendString(xbuf, I18N("The server provided the following challenge:"));
+    GWEN_Buffer_AppendString(xbuf, "\n");
+    GWEN_Buffer_AppendString(xbuf, challenge);
+  }
+
+  /* html version */
+  GWEN_Buffer_AppendString(xbuf,
+                           "<html>"
+                           "<p>");
+  snprintf(buffer, sizeof(buffer)-1,
+           I18N("Please enter the TAN for user <i>%s</i> at <i>%s</i>."), un, bn);
+  buffer[sizeof(buffer)-1]=0;
+  GWEN_Buffer_AppendString(xbuf, buffer);
+  GWEN_Buffer_AppendString(xbuf, "</p>");
+  if (challenge && *challenge) {
+    GWEN_Buffer_AppendString(xbuf, "<p>");
+    GWEN_Buffer_AppendString(xbuf, I18N("The server provided the following challenge:"));
+    GWEN_Buffer_AppendString(xbuf, "</p>"
+                             "<p align=\"center\" >"
+                             "<font color=\"blue\">");
+    GWEN_Buffer_AppendString(xbuf, challenge);
+    GWEN_Buffer_AppendString(xbuf,
+                             "</font>"
+                             "</p>"
+                             "</html>");
+  }
 
   nbuf=GWEN_Buffer_new(0, 256 ,0 ,1);
   AH_User_MkTanName(u, challenge, nbuf);
   rv=GWEN_Gui_GetPassword(GWEN_GUI_INPUT_FLAGS_TAN |
 			  /*GWEN_GUI_INPUT_FLAGS_NUMERIC |*/
-			  GWEN_GUI_INPUT_FLAGS_SHOW,
+                          GWEN_GUI_INPUT_FLAGS_SHOW,
 			  GWEN_Buffer_GetStart(nbuf),
-			  I18N("Enter TAN"),
-			  buffer,
-			  pwbuffer,
+                          I18N("Enter TAN"),
+                          GWEN_Buffer_GetStart(xbuf),
+                          pwbuffer,
 			  minLen,
 			  maxLen,
 			  0);
+  GWEN_Buffer_free(xbuf);
   GWEN_Buffer_free(nbuf);
   AB_BankInfo_free(bi);
   return rv;
@@ -1656,21 +1682,46 @@ void AH_User_LoadTanMethods(AB_USER *u) {
 	  }
 	  while(dbT) {
 	    AH_TAN_METHOD *tm;
+	    const char *s;
 
-	    tm=AH_TanMethod_fromDb(dbT);
-	    if (tm) {
-	      DBG_INFO(AQHBCI_LOGDOMAIN,
-			"Adding TAN method %d [%s] for GV version %d",
-                        AH_TanMethod_GetFunction(tm),
-			AH_TanMethod_GetMethodId(tm),
-			version);
-	      AH_TanMethod_SetGvVersion(tm, version);
-	      AH_TanMethod_List_Add(tm, ue->tanMethodDescriptions);
-	    }
-	    else {
-	      DBG_WARN(AQHBCI_LOGDOMAIN, "Invalid TAN method");
-	      GWEN_DB_Dump(dbT, 2);
-	    }
+	    tm=AH_TanMethod_new();
+	    AH_TanMethod_SetFunction(tm, GWEN_DB_GetIntValue(dbT, "function", 0, 0));
+	    AH_TanMethod_SetProcess(tm, GWEN_DB_GetIntValue(dbT, "process", 0, 0));
+	    AH_TanMethod_SetMethodId(tm, GWEN_DB_GetCharValue(dbT, "methodId", 0, 0));
+	    AH_TanMethod_SetMethodName(tm, GWEN_DB_GetCharValue(dbT, "methodName", 0, 0));
+	    AH_TanMethod_SetTanMaxLen(tm, GWEN_DB_GetIntValue(dbT, "tanMaxLen", 0, 0));
+	    AH_TanMethod_SetFormatId(tm, GWEN_DB_GetCharValue(dbT, "formatId", 0, 0));
+	    AH_TanMethod_SetPrompt(tm, GWEN_DB_GetCharValue(dbT, "prompt", 0, 0));
+	    AH_TanMethod_SetReturnMaxLen(tm, GWEN_DB_GetIntValue(dbT, "returnMaxLen", 0, 0));
+	    AH_TanMethod_SetMaxActiveLists(tm, GWEN_DB_GetIntValue(dbT, "maxActiveLists", 0, 0));
+	    AH_TanMethod_SetGvVersion(tm, GWEN_DB_GetIntValue(dbT, "gvVersion", 0, 0));
+	    s=GWEN_DB_GetCharValue(dbT, "multiTanAllowed", 0, NULL);
+	    if (s && strcasecmp(s, "j")==0)
+	      AH_TanMethod_SetMultiTanAllowed(tm, 1);
+	    AH_TanMethod_SetTimeShiftAllowed(tm, GWEN_DB_GetIntValue(dbT, "timeShiftAllowed", 0, 0));
+	    AH_TanMethod_SetTanListMode(tm, GWEN_DB_GetIntValue(dbT, "tanListMode", 0, 0));
+	    s=GWEN_DB_GetCharValue(dbT, "stornoAllowed", 0, NULL);
+	    if (s && strcasecmp(s, "j")==0)
+	      AH_TanMethod_SetStornoAllowed(tm, 1);
+	    s=GWEN_DB_GetCharValue(dbT, "needChallengeClass", 0, NULL);
+	    if (s && strcasecmp(s, "j")==0)
+	      AH_TanMethod_SetNeedChallengeClass(tm, 1);
+	    s=GWEN_DB_GetCharValue(dbT, "needChallengeAmount", 0, NULL);
+	    if (s && strcasecmp(s, "j")==0)
+	      AH_TanMethod_SetNeedChallengeAmount(tm, 1);
+	    AH_TanMethod_SetInitMode(tm, GWEN_DB_GetIntValue(dbT, "initMode", 0, 0));
+	    s=GWEN_DB_GetCharValue(dbT, "tanMediumIdNeeded", 0, NULL);
+	    if (s && strcasecmp(s, "j")==0)
+	      AH_TanMethod_SetNeedTanMediumId(tm, 1);
+	    AH_TanMethod_SetMaxActiveTanMedia(tm, GWEN_DB_GetIntValue(dbT, "maxActiveMedia", 0, 0));
+
+	    DBG_INFO(AQHBCI_LOGDOMAIN,
+		     "Adding TAN method %d [%s] for GV version %d",
+		     AH_TanMethod_GetFunction(tm),
+		     AH_TanMethod_GetMethodId(tm),
+		     version);
+	    AH_TanMethod_SetGvVersion(tm, version);
+	    AH_TanMethod_List_Add(tm, ue->tanMethodDescriptions);
 
 	    dbT=GWEN_DB_FindNextGroup(dbT, "tanMethod");
 	  }
