@@ -1,7 +1,4 @@
 /***************************************************************************
- $RCSfile$
-                             -------------------
-    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -22,10 +19,14 @@
 #include <gwenhywfar/directory.h>
 #include <gwenhywfar/xml.h>
 #include <gwenhywfar/fastbuffer.h>
-#include <gwenhywfar/io_file.h>
-#include <gwenhywfar/iomanager.h>
+#include <gwenhywfar/syncio_file.h>
 
 #include "../../../libs/aqbanking/types/bankinfo_l.h"
+
+#ifdef AQBANKING_WITH_PLUGIN_BACKEND_AQOFXCONNECT
+# include <aqofxconnect/user.h>
+#endif
+
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -53,15 +54,13 @@ int readCSVFile(const char *fname, const char *pname, GWEN_DB_NODE *db) {
   GWEN_DB_NODE *dbParams;
 
   dbParams=GWEN_DB_Group_new("params");
-  if (GWEN_DB_ReadFile(dbParams, pname,
-		       GWEN_DB_FLAGS_DEFAULT, 0, 2000)) {
+  if (GWEN_DB_ReadFile(dbParams, pname, GWEN_DB_FLAGS_DEFAULT)) {
     DBG_ERROR(0, "Error reading profile file \"%s\"", pname);
     GWEN_DB_Group_free(dbParams);
     return -1;
   }
 
-  if (GWEN_DB_ReadFileAs(db, fname, "csv", dbParams,
-			 GWEN_PATH_FLAGS_CREATE_GROUP, 0, 2000)) {
+  if (GWEN_DB_ReadFileAs(db, fname, "csv", dbParams, GWEN_PATH_FLAGS_CREATE_GROUP)) {
     DBG_ERROR(0, "Error reading data file \"%s\"", fname);
     GWEN_DB_Group_free(dbParams);
     return -1;
@@ -505,6 +504,18 @@ int readHBCIFile(const char *fname) {
     return -1;
   }
 
+  if (GWEN_DB_WriteFile(dbData,
+                        "out.conf",
+			GWEN_DB_FLAGS_QUOTE_VALUES | \
+                        GWEN_DB_FLAGS_WRITE_SUBGROUPS | \
+                        GWEN_DB_FLAGS_INDEND | \
+                        GWEN_DB_FLAGS_ADD_GROUP_NEWLINES | \
+                        GWEN_DB_FLAGS_ESCAPE_CHARVALUES | \
+			GWEN_DB_FLAGS_OMIT_TYPES)) {
+    DBG_ERROR(0, "Error writing bank file");
+    return -1;
+  }
+
   fprintf(stdout, "Updating database...\n");
   dbT=GWEN_DB_FindFirstGroup(dbData, "bank");
   while(dbT) {
@@ -540,6 +551,14 @@ int readHBCIFile(const char *fname) {
 		   strcmp(ver, "2.20")==0 ||
 		   strcmp(ver, "2.2.0")==0)
 	    ver="2.20";
+	  else if (strcmp(ver, "3")==0 ||
+		   strcmp(ver, "3.0")==0 ||
+		   strcmp(ver, "3.0.0")==0)
+	    ver="3.0";
+	  else if (strcmp(ver, "4")==0 ||
+		   strcmp(ver, "4.0")==0 ||
+		   strcmp(ver, "4.0.0")==0)
+	    ver="3.0";
 	}
 	if (addr) {
 	  const char *s;
@@ -616,6 +635,66 @@ int readHBCIFile(const char *fname) {
 	    AB_BankInfoService_List_Add(sv, AB_BankInfo_GetServices(bi));
 	  } /* if RDH5 */
 
+	  s=GWEN_DB_GetCharValue(dbT, "rdh6", 0, "nein");
+	  if (strcasecmp(s, "ja")==0) {
+	    AB_BANKINFO_SERVICE *sv;
+
+	    sv=AB_BankInfoService_new();
+	    AB_BankInfoService_SetType(sv, "HBCI");
+	    AB_BankInfoService_SetAddress(sv, addr);
+	    AB_BankInfoService_SetPversion(sv, ver);
+	    AB_BankInfoService_SetMode(sv, "RDH6");
+	    AB_BankInfoService_List_Add(sv, AB_BankInfo_GetServices(bi));
+	  } /* if RDH6 */
+
+	  s=GWEN_DB_GetCharValue(dbT, "rdh7", 0, "nein");
+	  if (strcasecmp(s, "ja")==0) {
+	    AB_BANKINFO_SERVICE *sv;
+
+	    sv=AB_BankInfoService_new();
+	    AB_BankInfoService_SetType(sv, "HBCI");
+	    AB_BankInfoService_SetAddress(sv, addr);
+	    AB_BankInfoService_SetPversion(sv, ver);
+	    AB_BankInfoService_SetMode(sv, "RDH7");
+	    AB_BankInfoService_List_Add(sv, AB_BankInfo_GetServices(bi));
+	  } /* if RDH7 */
+
+	  s=GWEN_DB_GetCharValue(dbT, "rdh8", 0, "nein");
+	  if (strcasecmp(s, "ja")==0) {
+	    AB_BANKINFO_SERVICE *sv;
+
+	    sv=AB_BankInfoService_new();
+	    AB_BankInfoService_SetType(sv, "HBCI");
+	    AB_BankInfoService_SetAddress(sv, addr);
+	    AB_BankInfoService_SetPversion(sv, ver);
+	    AB_BankInfoService_SetMode(sv, "RDH8");
+	    AB_BankInfoService_List_Add(sv, AB_BankInfo_GetServices(bi));
+	  } /* if RDH8 */
+
+	  s=GWEN_DB_GetCharValue(dbT, "rdh9", 0, "nein");
+	  if (strcasecmp(s, "ja")==0) {
+	    AB_BANKINFO_SERVICE *sv;
+
+	    sv=AB_BankInfoService_new();
+	    AB_BankInfoService_SetType(sv, "HBCI");
+	    AB_BankInfoService_SetAddress(sv, addr);
+	    AB_BankInfoService_SetPversion(sv, ver);
+	    AB_BankInfoService_SetMode(sv, "RDH9");
+	    AB_BankInfoService_List_Add(sv, AB_BankInfo_GetServices(bi));
+	  } /* if RDH9 */
+
+	  s=GWEN_DB_GetCharValue(dbT, "rdh10", 0, "nein");
+	  if (strcasecmp(s, "ja")==0) {
+	    AB_BANKINFO_SERVICE *sv;
+
+	    sv=AB_BankInfoService_new();
+	    AB_BankInfoService_SetType(sv, "HBCI");
+	    AB_BankInfoService_SetAddress(sv, addr);
+	    AB_BankInfoService_SetPversion(sv, ver);
+	    AB_BankInfoService_SetMode(sv, "RDH10");
+	    AB_BankInfoService_List_Add(sv, AB_BankInfo_GetServices(bi));
+	  } /* if RDH10 */
+
 	  s=GWEN_DB_GetCharValue(dbT, "pinTanUrl", 0, 0);
 	  if (s && *s) {
 	    AB_BANKINFO_SERVICE *sv;
@@ -702,9 +781,7 @@ int readATBLZFile2(const char *fname) {
                         GWEN_DB_FLAGS_INDEND | \
                         GWEN_DB_FLAGS_ADD_GROUP_NEWLINES | \
                         GWEN_DB_FLAGS_ESCAPE_CHARVALUES | \
-			GWEN_DB_FLAGS_OMIT_TYPES,
-			0,
-			2000)) {
+			GWEN_DB_FLAGS_OMIT_TYPES)) {
     DBG_ERROR(0, "Error writing bank file");
     return -1;
   }
@@ -956,7 +1033,16 @@ int readMSMFiles(const char *path,
 	    b=GWEN_XMLNode_FindFirstTag(b, "ProviderSettings", 0, 0);
 	  if (b) {
 	    const char *s;
-  
+	    int i;
+	    uint32_t uflags;
+
+	    uflags=0;
+	    i=GWEN_XMLNode_GetIntValue(b, "AcctListAvail", 0);
+#ifdef AQBANKING_WITH_PLUGIN_BACKEND_AQOFXCONNECT
+            if (i>0)
+	      uflags|=AO_USER_FLAGS_ACCOUNT_LIST;
+#endif
+
 	    s=readCharValueXml(b, "country", dbuf);
 	    if (s && *s) {
 	      if (strcasecmp(s, "USA")==0 ||
@@ -1027,7 +1113,8 @@ int readMSMFiles(const char *path,
 		pver=GWEN_XMLNode_GetCharValue(b, "OFXHeaderVer", 0);
 		server=readCharValueXml(b, "ProviderUrl",
 					dbuf);
-		if (pver && *pver && server && *server) {
+                if (pver && *pver && server && *server &&
+                    strcasecmp(server, "http://moneycentral.msn.com/cust404.htm")!=0) {
 		  AB_BANKINFO_SERVICE *sv;
   
 		  sv=AB_BankInfoService_new();
@@ -1186,8 +1273,7 @@ int readBcFile(const char *fname) {
 			GWEN_DB_FLAGS_INDEND | \
                         GWEN_DB_FLAGS_ADD_GROUP_NEWLINES | \
                         GWEN_DB_FLAGS_ESCAPE_CHARVALUES | \
-			GWEN_DB_FLAGS_OMIT_TYPES,
-			0, 2000)) {
+			GWEN_DB_FLAGS_OMIT_TYPES)) {
     DBG_ERROR(0, "Error writing bank file");
     return -1;
   }
@@ -1412,42 +1498,41 @@ int saveBankInfos(const char *path) {
   AB_BANKINFO *bi;
   uint32_t count=0;
   GWEN_BUFFER *xbuf;
-  GWEN_IO_LAYER *io;
-  int fd;
-  int err;
+  GWEN_SYNCIO *sio;
+  int rv;
   GWEN_FAST_BUFFER *fb;
 
   fprintf(stdout, "Saving database...\n");
-  fd=open(path, O_RDWR | O_CREAT | O_TRUNC,
-          S_IRUSR | S_IWUSR
-#ifdef S_IRGRP
-          |S_IRGRP
-#endif
-#ifdef S_IROTH
-          |S_IROTH
-#endif
-          );
-  if (fd==-1) {
-    DBG_ERROR(0, "open(%s): %s", path, strerror(errno));
-    return -1;
+  sio=GWEN_SyncIo_File_new(path, GWEN_SyncIo_File_CreationMode_CreateAlways);
+  GWEN_SyncIo_AddFlags(sio,
+		       GWEN_SYNCIO_FILE_FLAGS_READ |
+		       GWEN_SYNCIO_FILE_FLAGS_WRITE |
+		       GWEN_SYNCIO_FILE_FLAGS_UREAD |
+		       GWEN_SYNCIO_FILE_FLAGS_UWRITE);
+  rv=GWEN_SyncIo_Connect(sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_free(sio);
+    return rv;
   }
-  io=GWEN_Io_LayerFile_new(-1, fd);
-  GWEN_Io_Manager_RegisterLayer(io);
-  fb=GWEN_FastBuffer_new(512, io, 0, 2000);
 
-  GWEN_FASTBUFFER_WRITELINE(fb, err, "# This is an automatically created file");
-  if (err>=0) {
-    GWEN_FASTBUFFER_WRITELINE(fb, err, "# All banks are separated by newlines");
+  fb=GWEN_FastBuffer_new(512, sio);
+
+  GWEN_FASTBUFFER_WRITELINE(fb, rv, "# This is an automatically created file");
+  if (rv>=0) {
+    GWEN_FASTBUFFER_WRITELINE(fb, rv, "# All banks are separated by newlines");
   }
-  if (err>=0) {
-    GWEN_FASTBUFFER_WRITELINE(fb, err,
+  if (rv>=0) {
+    GWEN_FASTBUFFER_WRITELINE(fb, rv,
 			      "# Please do not modify this file, "
 			      "the index files rely on exact positions.");
   }
-  if (err<0) {
-    DBG_INFO(0, "here (%d)", err);
+  if (rv<0) {
+    DBG_INFO(0, "here (%d)", rv);
     DBG_ERROR(0, "Error writing bank file \"%s\"", path);
     GWEN_FastBuffer_free(fb);
+    GWEN_SyncIo_Disconnect(sio);
+    GWEN_SyncIo_free(sio);
     return -1;
   }
 
@@ -1495,17 +1580,19 @@ int saveBankInfos(const char *path) {
       DBG_ERROR(0, "Error writing bank file \"%s\"", path);
       GWEN_DB_Group_free(dbT);
       GWEN_FastBuffer_free(fb);
-      GWEN_Io_Layer_free(io);
+      GWEN_SyncIo_Disconnect(sio);
+      GWEN_SyncIo_free(sio);
       return -1;
     }
-    GWEN_FASTBUFFER_WRITELINE(fb, err, "");
-    if (err<0) {
-      DBG_ERROR_ERR(0, err);
+    GWEN_FASTBUFFER_WRITELINE(fb, rv, "");
+    if (rv<0) {
+      DBG_ERROR_ERR(0, rv);
       DBG_ERROR(0, "Error writing bank file \"%s\"", path);
       GWEN_DB_Group_free(dbT);
       GWEN_FastBuffer_free(fb);
-      GWEN_Io_Layer_free(io);
-      return err;
+      GWEN_SyncIo_Disconnect(sio);
+      GWEN_SyncIo_free(sio);
+      return rv;
     }
     GWEN_DB_Group_free(dbT);
 
@@ -1517,23 +1604,25 @@ int saveBankInfos(const char *path) {
   } /* while bi */
 
   /* flush fast buffer */
-  GWEN_FASTBUFFER_FLUSH(fb, err);
-  if (err<0) {
-    DBG_ERROR(0, "Error closing bank file \"%s\" (%d)", path, err);
+  GWEN_FASTBUFFER_FLUSH(fb, rv);
+  if (rv<0) {
+    DBG_ERROR(0, "Error closing bank file \"%s\" (%d)", path, rv);
     GWEN_FastBuffer_free(fb);
-    GWEN_Io_Layer_free(io);
-    return err;
+    GWEN_SyncIo_Disconnect(sio);
+    GWEN_SyncIo_free(sio);
+    return rv;
   }
   GWEN_FastBuffer_free(fb);
 
   /* flush and close io */
-  err=GWEN_Io_Layer_DisconnectRecursively(io, NULL, 0, 0, 30000);
-  if (err<0) {
-    DBG_ERROR(0, "Error closing bank file \"%s\" (%d)", path, err);
+  rv=GWEN_SyncIo_Disconnect(sio);
+  if (rv<0) {
+    DBG_ERROR(0, "Error closing bank file \"%s\" (%d)", path, rv);
     GWEN_FastBuffer_free(fb);
-    GWEN_Io_Layer_free(io);
-    return err;
+    GWEN_SyncIo_free(sio);
+    return rv;
   }
+  GWEN_SyncIo_free(sio);
 
   fprintf(stdout, "  Written %d banks.\n", count);
   return 0;
@@ -1587,9 +1676,7 @@ int makeBankInfos(const char *path) {
 			  GWEN_DB_FLAGS_INDEND | \
 			  GWEN_DB_FLAGS_ADD_GROUP_NEWLINES | \
 			  GWEN_DB_FLAGS_ESCAPE_CHARVALUES | \
-			  GWEN_DB_FLAGS_OMIT_TYPES,
-			  0,
-			  2000)) {
+			  GWEN_DB_FLAGS_OMIT_TYPES)) {
       DBG_ERROR(0, "Error writing bank file \"%s\"",
 		GWEN_Buffer_GetStart(dbuf));
       GWEN_DB_Group_free(dbT);
@@ -1630,21 +1717,21 @@ int makeBankInfos(const char *path) {
 
 int loadBanks(const char *fname, AB_BANKINFO_LIST *biList) {
   uint32_t count=0;
-  int fd;
-  GWEN_IO_LAYER *io;
+  GWEN_SYNCIO *sio;
   GWEN_FAST_BUFFER *fb;
+  int rv;
 
   fprintf(stdout, "Loading database, this will take a few minutes ...\n");
-  fd=open(fname, O_RDONLY | O_EXCL);
-  if (fd==-1) {
-    DBG_ERROR(0, "open(%s): %s", fname, strerror(errno));
-    return -1;
+  sio=GWEN_SyncIo_File_new(fname, GWEN_SyncIo_File_CreationMode_OpenExisting);
+  GWEN_SyncIo_AddFlags(sio, GWEN_SYNCIO_FILE_FLAGS_READ);
+  rv=GWEN_SyncIo_Connect(sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_free(sio);
+    return rv;
   }
 
-  io=GWEN_Io_LayerFile_new(fd, -1);
-  GWEN_Io_Manager_RegisterLayer(io);
-
-  fb=GWEN_FastBuffer_new(512, io, 0, 2000);
+  fb=GWEN_FastBuffer_new(512, sio);
 
   for(;;) {
     GWEN_DB_NODE *dbT;
@@ -1665,7 +1752,8 @@ int loadBanks(const char *fname, AB_BANKINFO_LIST *biList) {
       DBG_ERROR(0, "Could not load file \"%s\" (%d)", fname, rv);
       GWEN_DB_Group_free(dbT);
       GWEN_FastBuffer_free(fb);
-      GWEN_Io_Layer_free(io);
+      GWEN_SyncIo_Disconnect(sio);
+      GWEN_SyncIo_free(sio);
       return -1;
     }
 
@@ -1684,7 +1772,8 @@ int loadBanks(const char *fname, AB_BANKINFO_LIST *biList) {
   fprintf(stdout, "  Read %d banks.\n", count);
 
   GWEN_FastBuffer_free(fb);
-  GWEN_Io_Layer_free(io);
+  GWEN_SyncIo_Disconnect(sio);
+  GWEN_SyncIo_free(sio);
 
   return 0;
 }
@@ -1965,9 +2054,7 @@ int main(int argc, char **argv) {
       return 2;
     }
 
-    if (GWEN_DB_WriteFile(dbIdx,
-                          "index.conf.out",
-			  GWEN_DB_FLAGS_DEFAULT, 0, 2000)) {
+    if (GWEN_DB_WriteFile(dbIdx, "index.conf.out", GWEN_DB_FLAGS_DEFAULT)) {
       DBG_ERROR(0, "Error writing index file");
       return -1;
     }
